/* -------------------------------------------------------------

This file is a component of SDPA
Copyright (C) 2004-2020 SDPA Project

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

------------------------------------------------------------- */
/*--------------------------------------------------
  sdpa_right.h
--------------------------------------------------*/

#ifndef __sdpa_right_h__
#define __sdpa_right_h__

/*------------------------------------------
  Version Code Name

  SDPA 6 : Rosemary/2003Aug
  SDPA 7 : Margaret/2008Feb
  
------------------------------------------*/
  
static const char sdpa_right[] =
  "SDPA7 (Margaret/since 2008Feb) has been developed by SDPA Project.";
#ifdef VERSION
// VERSION is set by configure script
static const char sdpa_version[] = VERSION;
#else
// static const char sdpa_version[] = "7";
#endif

#endif // __sdpa_right_h__
